package se.liu.ida.tdde46;

import static io.restassured.RestAssured.*;
import static org.testng.Assert.*;

import org.apache.commons.lang3.RandomStringUtils;
import org.testng.annotations.Test;

import io.restassured.http.ContentType;
import io.restassured.response.Response;

public class PermissionTest {

	final static String HOST_NAME = "http://localhost:8080";

	@Test
	public void testAddedUnconstrainedPermissionIsValid() {
		String principalName = RandomStringUtils.randomAlphanumeric(20);
		String activityName = RandomStringUtils.randomAlphanumeric(20);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			put(HOST_NAME + "/principals/" + principalName);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"" + principalName + "\", \"activityName\": \"" + activityName + "\"}").
			post(HOST_NAME + "/permissions/create");

		Response response = given().
			accept(ContentType.ANY).
			get(HOST_NAME + "/permission/" + activityName + "/" + principalName);
		
		response.then().statusCode(200);
		assertTrue(response.asString().equals("true"));
	}

	@Test
	public void testNonExistingPermissionIsInvalid() {
		String principalName = RandomStringUtils.randomAlphanumeric(20);
		String activityName = RandomStringUtils.randomAlphanumeric(20);

		Response response = given().
			accept(ContentType.ANY).
			get(HOST_NAME + "/permission/" + activityName + "/" + principalName);
		
		response.then().statusCode(200);
		assertTrue(response.asString().equals("false"));
	}

	@Test
	public void testConstrained10sPermissionIsValidUntilAndOnlyUntilExpiration() {
		String principalName = RandomStringUtils.randomAlphanumeric(20);
		String activityName = RandomStringUtils.randomAlphanumeric(20);
		Integer durationInSeconds = 15;

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			put(HOST_NAME + "/principals/" + principalName);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"" + principalName + "\", \"activityName\": \"" + activityName + "\", \"durationInSeconds\": " + durationInSeconds + "}").
			post(HOST_NAME + "/permissions/create");
		
		try {
			Thread.sleep(1000 * (durationInSeconds - 5));
		} 
		catch (InterruptedException e) {
			fail(e.getMessage());
		}

		Response firstResponse = given().
			accept(ContentType.ANY).
			get(HOST_NAME + "/permission/" + activityName + "/" + principalName);
		
		firstResponse.then().statusCode(200);
		assertTrue(firstResponse.asString().equals("true"));

		try {
			Thread.sleep(1000 * 10);
		} 
		catch (InterruptedException e) {
			fail(e.getMessage());
		}

		Response secondResponse = given().
				accept(ContentType.ANY).
				get(HOST_NAME + "/permission/" + activityName + "/" + principalName);
			
		secondResponse.then().statusCode(200);
		assertTrue(secondResponse.asString().equals("false"));
	}
}