package se.liu.ida.tdde46;

import io.dropwizard.Application;
import io.dropwizard.setup.Bootstrap;
import io.dropwizard.setup.Environment;
import io.federecio.dropwizard.swagger.SwaggerBundle;
import io.federecio.dropwizard.swagger.SwaggerBundleConfiguration;
import se.liu.ida.tdde46.resources.PermissionResource;
import se.liu.ida.tdde46.resources.PermissionsResource;
import se.liu.ida.tdde46.resources.PingResource;
import se.liu.ida.tdde46.resources.PrincipalsResource;
import se.liu.ida.tdde46.types.PermissionsCollection;
import se.liu.ida.tdde46.types.PrincipalsCollection;

public class AuthenticatorApplication extends Application<AuthenticatorConfiguration> {
	public AuthenticatorApplication() {
	}

	public static void main(String[] args) throws Exception {
		new AuthenticatorApplication().run(args);
	}

	@Override
	public String getName() {
		return "TDDE46 Authenticator";
	}

	@Override
	public void initialize(Bootstrap<AuthenticatorConfiguration> bootstrap) {
		bootstrap.addBundle(new SwaggerBundle<AuthenticatorConfiguration>() {
			@Override
			protected SwaggerBundleConfiguration getSwaggerBundleConfiguration(AuthenticatorConfiguration configuration) {
				return configuration.swaggerBundleConfiguration;
			}
		});
	}

	@Override
	public void run(AuthenticatorConfiguration configuration, Environment environment) {
		final PrincipalsCollection principals = new PrincipalsCollection();
		final PermissionsCollection permissions = new PermissionsCollection(principals);
		
		environment.jersey().register(new PingResource());
		environment.jersey().register(new PrincipalsResource(principals));
		environment.jersey().register(new PermissionsResource(permissions));
		environment.jersey().register(new PermissionResource(permissions));
	}
}
