package se.liu.ida.tdde46.resources;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.codahale.metrics.annotation.Timed;
import com.google.common.base.Optional;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import se.liu.ida.tdde46.types.Permission;
import se.liu.ida.tdde46.types.PermissionsCollection;

@Api("/permission")
@Path("/permission")
@Produces(MediaType.APPLICATION_JSON)
public class PermissionResource {
    private final PermissionsCollection permissions;

    public PermissionResource(PermissionsCollection permissions) {
        this.permissions = permissions;
    }

    @GET
    @Path("/{name1}/{name2}")
    @ApiOperation(value="Evaluate permission.", notes="Evaluate whether the named principal is permitted to perform the named activity.")
    @ApiResponses(value = {
    		@ApiResponse(code = 200, message = "True if permitted, false is not permitted."),
    		@ApiResponse(code = 400, message = "Malformed request.")
    })
    @Timed
    public Response evaluatePermission(@PathParam("name1") Optional<String> activityName, @PathParam("name2") Optional<String> principalName) {
    	try {
			Thread.sleep(2000);
		} 
    	catch (InterruptedException e) {
    		return Response.status(500).entity("Thread interrupted.").build();
		}
    	
    	if(!activityName.isPresent())
			return Response.status(400).entity("Missing activityName in request.").build();

    	if(!principalName.isPresent())
			return Response.status(400).entity("Missing principalName in request.").build();
    	
    	
    	Optional<Permission> p = permissions.get(activityName.get(), principalName.get());
    	
    	if(!p.isPresent())
    		return Response.ok(false).build();
    	
    	if(p.get().getExpirationDate() == null)
    		return Response.ok(true).build();
    	
    	return Response.ok(p.get().getExpirationDate().isAfterNow()).build();
    }
}
