package se.liu.ida.tdde46.resources;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.codahale.metrics.annotation.Timed;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import se.liu.ida.tdde46.types.Permission.JsonRepresentation;
import se.liu.ida.tdde46.types.PermissionsCollection;
import se.liu.ida.tdde46.types.PermissionsCollection.MissingActivityNameInJsonRepresentationException;
import se.liu.ida.tdde46.types.PermissionsCollection.MissingPrincipalNameInJsonRepresentationException;
import se.liu.ida.tdde46.types.PermissionsCollection.UnrecognizedPrincipalException;

@Api("/permissions")
@Path("/permissions")
@Produces(MediaType.APPLICATION_JSON)
public class PermissionsResource {
    private final PermissionsCollection permissions;

    public PermissionsResource(PermissionsCollection permissions) {
        this.permissions = permissions;
    }

    @POST
    @Path("/create")
    @ApiOperation(value="Create a new permission.", notes="Creates a new permission for a specified action and a specified principal, with an optional duration.")
    @ApiResponses(value = {
    		@ApiResponse(code = 200, message = "Permission created."),
    		@ApiResponse(code = 400, message = "Failed to create permission.")
    })
    @Timed
    public Response postPermission(JsonRepresentation o) {
    	try {
    		return Response.ok(permissions.addFromJsonRepresentation(o)).build();
    	}
        catch(UnrecognizedPrincipalException e) {
        	return Response.status(400).entity("Unrecognized principalName.").build();
        }
    	catch(MissingPrincipalNameInJsonRepresentationException e) {
        	return Response.status(400).entity("Missing principalName in request body.").build();
    	}
    	catch(MissingActivityNameInJsonRepresentationException e) {
        	return Response.status(400).entity("Missing activityName in request body.").build();
    	}
    }

    @GET
    @Path("/")
    @ApiOperation(value="List existing permissions.", notes="Lists all existing permissions.")
    @ApiResponses(value = {
    		@ApiResponse(code = 200, message = "List of all existing permissions.")
    })
    @Timed
    public Response postPermission() {
        return Response.ok(permissions.getAll()).build();
    }
}
