package se.liu.ida.tdde46.resources;

import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.codahale.metrics.annotation.Timed;
import com.google.common.base.Optional;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import se.liu.ida.tdde46.types.Principal;
import se.liu.ida.tdde46.types.PrincipalsCollection;

@Api("/principals")
@Path("/principals")
@Produces(MediaType.APPLICATION_JSON)
public class PrincipalsResource {
    private final PrincipalsCollection principals;

    public PrincipalsResource(PrincipalsCollection principals) {
        this.principals = principals;
    }

    @PUT
    @Path("/{name}")
    @ApiOperation(value="Add a new principal to the authenticator.", notes="Once added, the principal may be assigned permissions. Principal name must not be blank.")
    @ApiResponses(value = {
    		@ApiResponse(code = 201, message = "Principal created."),
    		@ApiResponse(code = 204, message = "Principal updated."),
    		@ApiResponse(code = 418, message = "Failed to create/update principal.")
    })
    @Timed
    public Response putPrincipal(@PathParam("name") Optional<String> name) {
        if(!name.isPresent() || name.get().isBlank())
        	return Response.status(418).build();
        
        if(principals.add(name.get()))
        	return Response.status(201).build();
        else
        	return Response.status(204).build();
    }

    @GET
    @Path("/{name}")
    @ApiOperation(value="Fetch an existing principal.")
    @ApiResponses(value = {
    		@ApiResponse(code = 200, message = "Principal successfully retrieved.", response=Principal.class),
    		@ApiResponse(code = 404, message = "No matching principal found.")
    })
    @Timed
    public Response getPrincipal(@PathParam("name") Optional<String> name) {
        final Optional<Principal> principal = principals.get(name.orNull());
        if(principal.isPresent())
        	return Response.ok(principal.get()).build();
        else
        	return Response.status(404).build();
    }
}
