package se.liu.ida.tdde46.types;

import org.joda.time.DateTime;

import com.fasterxml.jackson.annotation.JsonProperty;

import se.liu.ida.tdde46.types.PermissionsCollection.MissingActivityNameInJsonRepresentationException;
import se.liu.ida.tdde46.types.PermissionsCollection.MissingPrincipalNameInJsonRepresentationException;

public class Permission {
	private final String principalName;
	private final String activityName;
	private final DateTime expirationDate;
	
	public Permission(final String principalName, final String activityName) {
		this(principalName, activityName, null);
	}
	
	public Permission(final String principalName, final String activityName, final DateTime expirationDate) {
		this.principalName = principalName;
		this.activityName = activityName;
		this.expirationDate = expirationDate;
	}

	public Permission(final JsonRepresentation o) {
		if(o.getPrincipalName() == null)
			throw new MissingPrincipalNameInJsonRepresentationException();
		
		if(o.getActivityName() == null)
			throw new MissingActivityNameInJsonRepresentationException();
		
		this.principalName = o.getPrincipalName();
		this.activityName = o.getActivityName();
		this.expirationDate = o.getDurationInSeconds() == null || o.getDurationInSeconds() < 1 ? null : DateTime.now().plusSeconds(o.getDurationInSeconds());
	}

	public String getPrincipalName() {
		return principalName;
	}

	public String getActivityName() {
		return activityName;
	}

	public DateTime getExpirationDate() {
		return expirationDate;
	}
	
	public boolean lastsLongerThan(Permission other) {
		if(other.getExpirationDate() == null) 
			return false;
		
		if(this.getExpirationDate() == null)
			return true;
		
		return this.getExpirationDate().isAfter(other.getExpirationDate());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((activityName == null) ? 0 : activityName.hashCode());
		result = prime * result + ((expirationDate == null) ? 0 : expirationDate.hashCode());
		result = prime * result + ((principalName == null) ? 0 : principalName.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Permission other = (Permission) obj;
		if (activityName == null) {
			if (other.activityName != null)
				return false;
		} else if (!activityName.equals(other.activityName))
			return false;
		if (expirationDate == null) {
			if (other.expirationDate != null)
				return false;
		} else if (!expirationDate.equals(other.expirationDate))
			return false;
		if (principalName == null) {
			if (other.principalName != null)
				return false;
		} else if (!principalName.equals(other.principalName))
			return false;
		return true;
	}
	
    public static class JsonRepresentation {
    	@JsonProperty("principalName")
    	private String principalName;
    	
    	@JsonProperty("activityName")
    	private String activityName;
    	
    	@JsonProperty("durationInSeconds")
    	private Integer durationInSeconds;
    	
    	public String getPrincipalName() {
    		return principalName;
    	}

    	public String getActivityName() {
    		return activityName;
    	}

    	public Integer getDurationInSeconds() {
    		return durationInSeconds;
    	}
	}
}
