package se.liu.ida.tdde46.types;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.google.common.base.Optional;

import se.liu.ida.tdde46.types.Permission.JsonRepresentation;

@SuppressWarnings("serial")
public class PermissionsCollection {
	private final List<Permission> permissions = Collections.synchronizedList(new ArrayList<Permission>());
	private final PrincipalsCollection principals;
	
	public PermissionsCollection(final PrincipalsCollection principals) {
		this.principals = principals;
	}
	
	public Permission add(final Permission newPermission) throws UnrecognizedPrincipalException {
		if(!principals.isPresent(newPermission.getPrincipalName()))
			throw new UnrecognizedPrincipalException(newPermission.getPrincipalName());
		
		Optional<Permission> existingPermission = get(newPermission.getActivityName(), newPermission.getPrincipalName());
		if(!existingPermission.isPresent()) {
			permissions.add(newPermission);
			return newPermission;
		}
		
		if(newPermission.lastsLongerThan(existingPermission.get())) {
			permissions.remove(existingPermission.get());
			permissions.add(newPermission);
			return newPermission;
		}
		
		return existingPermission.get();
		
	}

	public List<Permission> getAll() {
		return new ArrayList<Permission>(permissions);
	}
	
	public Optional<Permission> get(final String activityName, final String principalName) {
		for(Permission p : permissions) {
			if(p.getActivityName().equals(activityName) && p.getPrincipalName().equals(principalName))
				return Optional.of(p);
		}
		
		return Optional.absent();
	}
	
	public Permission addFromJsonRepresentation(final JsonRepresentation o) {
		return add(new Permission(o));
	}

	public static class UnrecognizedPrincipalException extends IllegalArgumentException {
		public UnrecognizedPrincipalException(final String principalName) {
			super("The named principal \"" + principalName + "\" could not be found.");
		}
	}

	public static class MissingPrincipalNameInJsonRepresentationException extends IllegalArgumentException {
		public MissingPrincipalNameInJsonRepresentationException() {
			super("Mandatory principalName property missing.");
		}
	}

	public static class MissingActivityNameInJsonRepresentationException extends IllegalArgumentException {
		public MissingActivityNameInJsonRepresentationException() {
			super("Mandatory activityName property missing.");
		}
	}
}
